/*
 *	filer.h
 */

#ifndef _FILER_H_
#define _FILER_H_

#include <gba.h>
#include <font8.h>
#include <gbafs.h>
#include <xfont.h>
#include <chartype.h>
#include <scrn.h>

#include "bookmark.h"
#include "bitmap.h"
#include "colors.h"
#include "fonts.h"
#include "ini.h"
#include "keymap.h"
#include "listbox.h"
#include "options.h"
#include "screen.h"
#include "timer.h"
#include "token.h"
#include "win.h"

/*
 *	Constants
 */

#define KEY_REPEAT	0x4000

/*
 *	SRAM space definitions
 */

#define BOOKMARK_ADDR	SRAM_ADDR
#define BOOKMARK_SIZE	(sizeof(BMHEAD) + sizeof(BMENTRY) * MAX_BOOKMARKS)

#define FONTINFO_ADDR	(BOOKMARK_ADDR + BOOKMARK_SIZE)
#define FONTINFO_SIZE	(sizeof(FONTINFO))

#define SCRNDIR_ADDR	(FONTINFO_ADDR + FONTINFO_SIZE)
#define SCRNDIR_SIZE	(sizeof(SCRNDIR))

#define SCRNCOLORS_ADDR	(SCRNDIR_ADDR + SCRNDIR_SIZE)
#define SCRNCOLORS_SIZE	(sizeof(SCRNCOLORS))

/*
 *	Structures
 */

/*
 *	Globals
 */

extern const BITMAP	bmFolder;
extern const BITMAP	bmBookmark;

extern const u8		sVersion[];
extern const u8		sCopyright[];
extern const u8		sPreviewSample[];
extern const u8		sPreviewKanjiSample[];

/*
 *	External functions
 */

/* plugins.c */
BOOL RunPlugin(PDIRENT pdir, PDIRENT pfile);

/* sram.c */
void ReadSRAMByte(vu8 *sram, u8 *pv, int count);
void WriteSRAMByte(vu8 *sram, u8 *pv, int count);
u8 ReadSRAM8(vu8 *sram);
u16 ReadSRAM16(vu8 *sram);
u32 ReadSRAM32(vu8 *sram);
void WriteSRAM8(vu8 *sram, u8 v);
void WriteSRAM16(vu8 *sram, u16 v);
void WriteSRAM32(vu8 *sram, u32 v);
int CompSRAM16(vu8 *sram, u16 *pv);
int CompSRAM32(vu8 *sram, u32 *pv);
int CompSRAMByte(vu8 *sram, u8 *pv, int count);

/* subr.c */
int GetKey();
void InitKey();
int PutStr(int x, int y, u8 *s);
int Printf(int x, int y, u8 *fmt, ...);
int LeftPrintf(int x, int y, int w, u8 *fmt, ...);
int RightPrintf(int x, int y, int w, u8 *fmt, ...);
int _strnicmp(u8 *ps, u8 *pd, int len);

/* timer.c*/
void InitTimer();
void Wait(u32 msec);

/* view.c */
void StartView(PDIRENT pdir);

/* viewjpeg.c */
BOOL ViewJpeg(PDIRENT pdir, PDIRENT pfile);

/* viewtext.c */
int ViewTextGoto(int pageno);
void ViewText(PDIRENT pdir);

/* help.c */
int ViewHelp(u8 *title);

#endif // _FILER_H_

