/*
 *	help.c
 */

#include "filer.h"

typedef struct _MSGS {
	const u8	*key;
	const u8	*desc;
} MSGS, *PMSGS;

typedef struct _HELPMSG {
	u8	*title;
	int	nmsgs;
	PMSGS	pmsgs;
} HELPMSG, *PHELPMSG;

static const MSGS	aMsgs1[] = {
	{ "Left",	"Previous page" },
	{ "Right",	"Next page" },
	{ "Up",		"Previous item" },
	{ "Down",	"Next item" },
	{ "A",		"Select item" },
	{ "B",		"Back to previous directory" },
	{ "L",		"-" },
	{ "R",		"-" },
	{ "Select",	"Options" },
	{ "Start",	"-" },
};

static const MSGS	aMsgs2[] = {
	{ "Left",	"Previous page" },
	{ "Right",	"Next page" },
	{ "Up",		"Previous page" },
	{ "Down",	"Next page" },
	{ "A",		"Next page" },
	{ "B",		"Back to filer" },
	{ "L",		"Bookmark" },
	{ "R",		"Previous page" },
	{ "Select",	"Options" },
	{ "Start",	"-" },
};

static const MSGS	aMsgs3[] = {
	{ "Left",	"Move left" },
	{ "Right",	"Move right" },
	{ "Up",		"Move up" },
	{ "Down",	"Move down" },
	{ "A",		"-" },
	{ "B",		"Back to filer" },
	{ "L",		"Previous JPEG image" },
	{ "R",		"Next JPEG image" },
	{ "Select",	"-" },
	{ "Start",	"-" },
};

static const MSGS	aMsgs4[] = {
	{ NULL,		sVersion },
	{ NULL,		"" },
	{ NULL,		sCopyright },
};

static const HELPMSG	aHelpMsg[] = {
	{ "Filer mode",	arraysizeof(aMsgs1), (PMSGS)aMsgs1 },
	{ "Text mode",	arraysizeof(aMsgs2), (PMSGS)aMsgs2 },
	{ "Image mode",	arraysizeof(aMsgs3), (PMSGS)aMsgs3 },
	{ "About",	arraysizeof(aMsgs4), (PMSGS)aMsgs4 },
};

#define nHelpMsg	((int)arraysizeof(aHelpMsg))

/*
 *	ViewHelpMsg
 */
static void ViewHelpMsg(u8 *title, int page)
{
	u8		buf[64];
	PHELPMSG	phelp;
	PMSGS		pmsgs;
	int		w, h;
	int		x, y;
	int		i;
	RECT		rc;
	RECT		rcv;

	XfontSelectFontObject(FO_SMALL);
	w = XfontGetMaxCharWidth(' ') * 10;
	h = XfontGetCharHeight();

	GetClientRect(&rc);

	phelp = (PHELPMSG)&aHelpMsg[page];
	_sprintf(buf, "%s - %s", title, phelp->title);
	SetWinTitle(buf);
	ShowWinPages(page + 1, nHelpMsg);

	x = rc.x;
	y = rc.y;
	for (i = 0; i < phelp->nmsgs; i++) {
		pmsgs = &phelp->pmsgs[i];
		if (pmsgs->key) {
			LeftPrintf(x, y, w, "%s", pmsgs->key);
			LeftPrintf(x + w, y, rc.w - w, "%s", pmsgs->desc);
		} else
			LeftPrintf(x, y, rc.w, "%s", pmsgs->desc);
		y += h;
	}

	EraseScrn(rc.x, y, rc.w, rc.h - (y - rc.y));

#if 0
	rcv.x = rc.x;
	rcv.y = rc.y + rc.h - h * 2;
	rcv.w = rc.w;
	rcv.h = h * 2;

	LeftPrintf(rcv.x, rcv.y, rcv.w, "%s", sVersion);
	LeftPrintf(rcv.x, rcv.y + h, rcv.w, "%s", sCopyright);
#endif
}

/*
 *	ViewHelp
 */
int ViewHelp(u8 *title)
{
	CREATEWIN	cw;
	int		page;
	int		key;

	/*
	 *	Put frame
	 */

	memset(&cw, 0, sizeof cw);
	CreateWinFrame(&cw);

	/*
	 *	Put help
	 */

	page = 0;
	for (;;) {
		ViewHelpMsg(title, page);
		while (!(key = GetKey()))
			;
		switch (GetKeyFunc(key)) {
		case KF_OPTIONS:
		case KF_BACK:
			return LB_CANCEL;
		case KF_NEXTPAGE:
		case KF_NEXTITEM:
			if (page + 1 >= nHelpMsg)
				page = 0;
			else
				page++;
			break;
		case KF_PREVPAGE:
		case KF_PREVITEM:
			if (page <= 0)
				page = nHelpMsg - 1;
			else
				page--;
			break;
		}
	}

	return LB_CANCEL;
}

