/*
 *	listbox.c
 */

#include "filer.h"

typedef struct _LISTOBJ {
	PLISTBOX	plb;
	int		nitemspp;
	int		npages;
	int		curpage;
	int		curindex;
	int		h;
} LISTOBJ, *PLISTOBJ;

/*
 *	InitListBox
 */
static int InitListBox(PLISTOBJ plo)
{
	PLISTBOX	plb;
	PRECT		prc;

	plb = plo->plb;
	prc = &plb->rc;

	XfontSelectFontObject(FO_SMALL);
	plo->h = XfontGetCharHeight();

	plo->nitemspp = plb->rc.h / plo->h;
	if (plo->nitemspp == 0)
		return LB_ERROR;
	plo->npages = (plb->nitems + plo->nitemspp - 1) / plo->nitemspp;
	plo->curindex = plb->defitem;
	plo->curpage = plo->curindex / plo->nitemspp;

	return 0;
}

/*
 *	DoCallback
 */
void DoCallback(PLISTOBJ plo)
{
	PLISTBOX	plb;

	plb = plo->plb;
	if (plb->callback)
		plb->callback(plo->curindex, plb->params);
}

/*
 *	ViewListBox
 */
static void ViewListBox(PLISTOBJ plo)
{
	PLISTBOX	plb;
	PRECT		prc;
	int		i, n, y;

	plb = plo->plb;
	prc = &plb->rc;
	i = plo->curpage * plo->nitemspp;
	n = min(plb->nitems, plo->nitemspp + i);
	y = prc->y;

	XfontSelectFontObject(FO_SMALL);
	EraseScrn(prc->x, prc->y, prc->w, prc->h);
	for (; i < n; i++) {
		LeftPrintf(prc->x, y, prc->w, "%s", plb->items[i]);
		y += plo->h;
	}

	ShowWinPages(plo->curpage + 1, plo->npages);
}

/*
 *	ViewNewList
 */
static void ViewNewList(PLISTOBJ plo)
{
	int		p;

	p = plo->curindex / plo->nitemspp;
	if (p != plo->curpage) {
		plo->curpage = p;
		ViewListBox(plo);
	}
}

/*
 *	ViewUp
 */
static void ViewUp(PLISTOBJ plo)
{
	PLISTBOX	plb;

	plb = plo->plb;
	if (!plb->nitems)
		return;
	if (plo->curindex <= 0)
		plo->curindex = plb->nitems - 1;
	else
		plo->curindex--;
	ViewNewList(plo);
	DoCallback(plo);
}

/*
 *	ViewDown
 */
static void ViewDown(PLISTOBJ plo)
{
	PLISTBOX	plb;

	plb = plo->plb;
	if (!plb->nitems)
		return;
	if (plo->curindex >= plb->nitems - 1)
		plo->curindex = 0;
	else
		plo->curindex++;
	ViewNewList(plo);
	DoCallback(plo);
}

/*
 *	ViewLeft
 */
static void ViewLeft(PLISTOBJ plo)
{
	PLISTBOX	plb;

	plb = plo->plb;
	if (!plb->nitems)
		return;
	if (plo->npages <= 1) {
		plo->curindex = 0;
		DoCallback(plo);
		return;
	}
	if (plo->curpage <= 0)
		plo->curpage = plo->npages - 1;
	else
		plo->curpage--;
	plo->curindex = plo->curpage * plo->nitemspp;
	ViewListBox(plo);
	DoCallback(plo);
}

/*
 *	ViewRight
 */
static void ViewRight(PLISTOBJ plo)
{
	PLISTBOX	plb;

	plb = plo->plb;
	if (!plb->nitems)
		return;
	if (plo->npages <= 1) {
		plo->curindex = plb->nitems - 1;
		DoCallback(plo);
		return;
	}
	if (plo->curpage >= plo->npages - 1)
		plo->curpage = 0;
	else
		plo->curpage++;
	plo->curindex = plo->curpage * plo->nitemspp;
	ViewListBox(plo);
	DoCallback(plo);
}

/*
 *	ListBox
 */
int ListBox(PLISTBOX plb)
{
	int		x, y;
	int		ix;
	int		key;
	int		res;
	CREATEWIN	cw;
	PRECT		prc;
	LISTOBJ		lo;
	PLISTOBJ	plo;

	/*
	 *	Put the listbox
	 */

	memset(&cw, 0, sizeof cw);
	CreateWinFrame(&cw);
	SetWinTitle(plb->title);

	plo = &lo;
	plo->plb = plb;
	res = InitListBox(plo);
	if (res)
		return res;

	prc = &plb->rc;

	ViewListBox(plo);
	DoCallback(plo);

	/*
	 *	Select item
	 */

	for (;;) {
		ix = plo->curindex % plo->nitemspp;
		ReverseScrn(prc->x, prc->y + (ix * plo->h), prc->w, plo->h);
		while (!(key = GetKey()))
			;
		ReverseScrn(prc->x, prc->y + (ix * plo->h), prc->w, plo->h);

		switch (GetKeyFunc(key)) {
		case KF_OPTIONS:
		case KF_BACK:
			return LB_CANCEL;
		case KF_OK:
			goto out;
		case KF_PREVITEM:
			ViewUp(plo);
			break;
		case KF_NEXTITEM:
			ViewDown(plo);
			break;
		case KF_PREVPAGE:
			ViewLeft(plo);
			break;
		case KF_NEXTPAGE:
			ViewRight(plo);
			break;
		}
	}

out:
	return lo.curindex;
}

