/*
 *	token.c
 */

#include "filer.h"

//#define DEBUG

#ifdef DEBUG
/*
 *	DebugToken
 */
static void DebugToken(int t, PTOKEN ptok)
{
	u8	buf[80];
	int	i;

	ClearScrn();
	switch (t) {
	default:
		Fix8printf(0, 0, "Type: %d: '%c'(0x%x)", t, t, t);
		break;
	case T_EOF:
		Fix8printf(0, 0, "Type: T_EOF");
		break;
	case T_NAME:
		Fix8printf(0, 0, "Type: T_NAME");
		for (i = 0; i < ptok->len; i++)
			buf[i] = ((u8 *)(ptok->pvalue))[i];
		buf[i] = 0;
		Fix8printf(0, 8, "Value: %s", buf);
		break;
	case T_VALUE:
		Fix8printf(0, 0, "Type: T_VALUE");
		Fix8printf(0, 8, "Value: %d(%x)", ptok->pvalue, ptok->pvalue);
		break;
	}
	Fix8printf(0, 16, "pcur:0x%x", ptok->pcur);
	while (GetKey() == 0) ;
}
#endif

/*
 *	toupper
 */
u8 toupper(u8 c)
{
	return (c >= 'a' && c <= 'z') ? (c - ('a' - 'A')) : c;
}

/*
 *	isdigit
 */
static BOOL isdigit(u8 c)
{
	return (c >= '0' && c <= '9');
}

/*
 *	ishex
 */
static BOOL ishex(u8 c)
{
	return (c >= 'A' && c <= 'F') || (c >= 'a' && c <= 'f');
}

/*
 *	isxdigit
 */
static BOOL isxdigit(u8 c)
{
	return isdigit(c) || ishex(c);
}

/*
 *	isname
 */
static BOOL isname(u8 c)
{
	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c == '_');
}

/*
 *	isnamenum
 */
static BOOL isnamenum(u8 c)
{
	return isname(c) || isdigit(c);
}

/*
 *	atoi
 */
static int atoi(u8 **pp, u8 *pe)
{
	u8		c;
	int		x;

	x = 0;
	while (*pp < pe) {
		c = **pp;
		if (isdigit(c))
			x = x * 10 + (c - '0');
		else
			goto out;
		(*pp)++;
	}

out:
	return x;
}

/*
 *	atox
 */
static int atox(u8 **pp, u8 *pe)
{
	u8		c;
	int		x;

	x = 0;
	while (*pp < pe) {
		c = toupper(**pp);
		if (isdigit(c))
			x = x * 16 + (c - '0');
		else if (ishex(c))
			x = x * 16 + (c - 'A' + 10);
		else
			goto out;
		(*pp)++;
	}

out:
	return x;
}

/*
 *	GetToken
 */
int GetToken(PTOKEN ptok)
{
	int		type;
	u8		c;
	u8		*pcur;
	u8		*pend;
	u8		*pval;

	pcur = pval = ptok->pcur;
	pend = ptok->pend;

	/*
	 *	skip blanks/comments
	 */

	for (;;) {
		if (pcur >= pend) {
			type = T_EOF;
			goto out;
		}

		switch (*pcur) {
		case ' ':
		case '\t':
		case '\r':
			pcur++;
			continue;
		case '\n':
			type = *pcur;
			pcur++;
			ptok->curline++;
			goto out;
		case ';':
			while (pcur < pend && *pcur != '\n')
				pcur++;
			continue;
		}
		break;
	}

	/*
	 *	token
	 */

	pval = pcur;
	c = *pcur;
	if (isdigit(c)) {
		type = T_VALUE;
		if (&pcur[2] <= pend && c == '0' && pcur[1] == 'x') {
			pcur += 2;
			if (pcur >= pend || !isxdigit(*pcur)) {
				type = T_ERROR;
				goto out;
			}
			ptok->pvalue = (void *)atox(&pcur, pend);
		} else
			ptok->pvalue = (void *)atoi(&pcur, pend);
	} else if (isname(c)) {
		type = T_NAME;
		ptok->pvalue = pval;
		while (pcur < pend && isnamenum(*pcur))
			pcur++;
	} else {
		type = c;
		pcur++;
	}

out:
	ptok->pcur = pcur;
	ptok->len = pcur - pval;
#ifdef DEBUG
	DebugToken(type, ptok);
#endif

	return type;
}

/*
 *	InitToken
 */
void InitToken(PTOKEN ptok, u8 *ptop, u8 *pend)
{
	ptok->ptop = ptok->pcur = ptop;
	ptok->pend = pend;
	ptok->curline = 1;
}

