/*
 *	view.c
 */

#include "filer.h"

static int	Wsmall, Hsmall;

static RECT	rcFile;
static RECT	rcIcon;
static RECT	rcName;

static int	nFilesPP;

/*
 *	view object
 */

typedef struct _VIEWOBJ {
	struct _VIEWOBJ	*ppv;
	PDIRENT		pdir;
	u8		*ppath;
	u8		*pcurpath;
	int		nfiles;
	int		npages;
	int		curpage;
	int		curindex;
} VIEWOBJ, *PVIEWOBJ;

//========================

/*
 *	InitPositions
 */
static void InitPositions()
{
	int		hsmall;
	int		wk;

	XfontSelectFontObject(FO_SMALL);
	hsmall = XfontGetCharHeight();

	wk = XfontGetMaxCharWidth(0x81);

	GetClientRect(&rcFile);

	rcIcon.x = rcFile.x + (wk - bmFolder.width) / 2;
	rcIcon.y = rcFile.y + (hsmall - bmFolder.height) / 2;
	rcIcon.w = bmFolder.width;
	rcIcon.h = bmFolder.height;

	rcName.x = rcFile.x + wk;
	rcName.y = rcFile.y;
	rcName.w = rcFile.w - wk;
	rcName.h = hsmall;

	nFilesPP = rcFile.h / hsmall;
}

/*
 *	ViewAddPath
 */
void ViewAddPath(PVIEWOBJ pv)
{
	PDIRENT		pdir;
	int		len;

	pdir = pv->pdir;
	if (pdir->namesize) {
		len = strlen(pdir->fileaddr);
		*pv->pcurpath++ = '\\';
		strcpy(pv->pcurpath, pdir->fileaddr);
		pv->pcurpath += len;
	} else
		pv->pcurpath[0] = '\0';
}

/*
 *	ViewInit
 */
static void ViewInit(PVIEWOBJ pv)
{
	PDIRENT		pdir;

	pdir = pv->pdir;
	pv->nfiles = pdir->filesize / sizeof(*pdir);
	pv->npages = (pv->nfiles + nFilesPP - 1) / nFilesPP;
	pv->curpage = 0;
	pv->curindex = 0;
}

/*
 *	ViewFiles
 */
static void ViewFiles(PVIEWOBJ pv)
{
	int		i, n, x, y, yi;
	PDIRENT		pdir;

	y = rcName.y;
	yi = rcIcon.y;
	i = pv->curpage * nFilesPP;
	n = min(pv->nfiles, nFilesPP + i);
	pdir = (PDIRENT)(pv->pdir->fileaddr + pv->pdir->namesize) + i;

	ShowWinPages(pv->curpage + 1, pv->npages);

	for (; i < n; i++, pdir++) {
		if (pdir->attr & DA_DIR)
			CopyBitmap16((PBITMAP)&bmFolder, rcIcon.x, yi, 0);
		else
			EraseScrn(rcIcon.x, yi, rcIcon.w, rcIcon.h);
		LeftPrintf(rcName.x, y, rcName.w, "%s", pdir->fileaddr);
		y += rcName.h;
		yi += rcName.h;
	}

	EraseScrn(rcFile.x, y, rcFile.w, (rcFile.y + rcFile.h) - y);
}

/*
 *	ViewNewPage
 */
static void ViewNewPage(PVIEWOBJ pv)
{
	int		p;

	p = pv->curindex / nFilesPP;
	if (p != pv->curpage) {
		pv->curpage = p;
		ViewFiles(pv);
	}
}

/*
 *	ViewUp
 */
static void ViewUp(PVIEWOBJ pv)
{
	if (!pv->nfiles)
		return;
	if (pv->curindex <= 0)
		pv->curindex = pv->nfiles - 1;
	else
		pv->curindex--;
	ViewNewPage(pv);
}

/*
 *	ViewDown
 */
static void ViewDown(PVIEWOBJ pv)
{
	if (!pv->nfiles)
		return;
	if (pv->curindex >= pv->nfiles - 1)
		pv->curindex = 0;
	else
		pv->curindex++;
	ViewNewPage(pv);
}

/*
 *	ViewLeft
 */
static void ViewLeft(PVIEWOBJ pv)
{
	if (!pv->nfiles)
		return;
	if (pv->npages <= 1) {
		pv->curindex = 0;
		return;
	}
	if (pv->curpage <= 0)
		pv->curpage = pv->npages - 1;
	else
		pv->curpage--;
	pv->curindex = pv->curpage * nFilesPP;
	ViewFiles(pv);
}

/*
 *	ViewRight
 */
static void ViewRight(PVIEWOBJ pv)
{
	if (!pv->nfiles)
		return;
	if (pv->npages <= 1) {
		pv->curindex = pv->nfiles - 1;
		return;
	}
	if (pv->curpage >= pv->npages - 1)
		pv->curpage = 0;
	else
		pv->curpage++;
	pv->curindex = pv->curpage * nFilesPP;
	ViewFiles(pv);
}

/*
 *	RefreshView
 */
static void RefreshView(PVIEWOBJ pv, BOOL force)
{
	CREATEWIN	cw;

	if (force) {
		memset(&cw, 0, sizeof cw);
		CreateWinFrame(&cw);
		InitPositions();
	}
	SetWinTitle(pv->ppath);
	ViewFiles(pv);
}

static void DoView(PVIEWOBJ pv);

/*
 *	ViewEnter
 */
static void ViewEnter(PVIEWOBJ pv)
{
	PDIRENT		pdir;

	if (!pv->nfiles)
		return;
	pdir = (PDIRENT)(pv->pdir->fileaddr + pv->pdir->namesize) +
		pv->curindex;
	if (pdir->attr & DA_DIR) {
		VIEWOBJ		vo;

		vo.ppv = pv;
		vo.ppath = pv->ppath;
		vo.pcurpath = pv->pcurpath;
		vo.pdir = pdir;
		DoView(&vo);
		*pv->pcurpath = '\0';
		RefreshView(pv, FALSE);
	} else {
		if (ViewBMP(pdir))
			;
		else if (ViewJpeg(pv->pdir, pdir))
			;
		else if (RunPlugin(pv->pdir, pdir))
			;
		else
			ViewText(pdir);
		RefreshView(pv, TRUE);
	}
}

/*
 *	DoView
 */
static void DoView(PVIEWOBJ pv)
{
	int		y;
	int		key;
	int		mode;

	ViewAddPath(pv);
	ViewInit(pv);
	RefreshView(pv, FALSE);

	for (;;) {
		y = (pv->curindex - pv->curpage * nFilesPP) * rcName.h +
			rcName.y;
		ReverseScrn(rcName.x, y, rcName.w, rcName.h);
		while ((key = GetKey()) == 0)
			;
		ReverseScrn(rcName.x, y, rcName.w, rcName.h);

		switch (GetKeyFunc(key)) {
		case KF_PREVITEM:
			ViewUp(pv);
			break;
		case KF_NEXTITEM:
			ViewDown(pv);
			break;
		case KF_PREVPAGE:
			ViewLeft(pv);
			break;
		case KF_NEXTPAGE:
			ViewRight(pv);
			break;
		case KF_OK:
			ViewEnter(pv);
			break;
		case KF_BACK:
			if (pv->ppv)
				return;
			break;
		case KF_OPTIONS:
			if (ViewOptions() >= 0)
				ViewInit(pv);
			RefreshView(pv, TRUE);
			break;
		}
	}
}

/*
 *	StartView
 */
void StartView(PDIRENT pdir)
{
	VIEWOBJ		vo;
	u8		pathbuf[256];
	CREATEWIN	cw;

	memset(&cw, 0, sizeof cw);
	CreateWinFrame(&cw);
	InitPositions();

	vo.ppath = pathbuf;
	vo.pcurpath = vo.ppath;
	vo.ppv = NULL;
	vo.pdir = pdir;

	DoView(&vo);
}

