/*
 *	viewtext.c
 */

#include "filer.h"

#define INVALID_PAGENO	(0xffffffff)
#define INVALID_OFFSET	(0xffffffff)

/*
 *	variables
 */

static int	Hnormal;

static RECT	rcMark;
static RECT	rcText;

static u32	CurPage;
static u32	BookmarkPage;

static u32	*pPage;
static u32	*pPageEnd;

static u32	nPages;
static u32	workPage;
static u32	workOffset;
static u32	StartPage;
static u32	StartOffset;
static u8	PageStat;

#define PS_FORW		0x01
#define PS_BACK		0x02
#define PS_DONE		0x03

static u8	*pTextName;
static u8	*pText;
static u32	nText;

/*
 *	InitPositions
 */
static void InitPositions()
{
	int		wk;
	int		hsmall;
	RECT		rcwin;

	XfontSelectFontObject(FO_SMALL);
	hsmall = XfontGetCharHeight();

	wk = XfontGetMaxCharWidth(0x81);

	XfontSelectFontObject(FO_NORMAL);
	Hnormal = XfontGetCharHeight();

	GetClientRect(&rcText);
	GetWindowRect(&rcwin);

	rcMark.x = rcwin.x + (wk - bmBookmark.width) / 2;
	rcMark.y = rcwin.y + (hsmall - bmBookmark.height) / 2;
	rcMark.w = bmBookmark.width;
	rcMark.h = bmBookmark.height;
}

/*
 *	ViewBookmark
 */
static void ViewBookmark()
{
	if (BookmarkPage != INVALID_PAGENO && BookmarkPage == CurPage)
		CopyBitmap16((PBITMAP)&bmBookmark, rcMark.x, rcMark.y, 0);
	else
		EraseScrn(rcMark.x, rcMark.y, rcMark.w, rcMark.h);
}

/*
 *	ViewPageNumber
 */
static void ViewPageNumber()
{
	u32		pageno;

	pageno = (PageStat == PS_DONE) ? nPages : workPage;
	ShowWinPages(CurPage + 1, pageno);
}

/*
 *	InitPage
 */
static void InitPage(u32 offset)
{
	pPage = (u32 *)EWRAM_ADDR;
	pPageEnd = (u32 *)(EWRAM_ADDR + EWRAM_SIZE);

	StartPage = workPage = CurPage;
	StartOffset = workOffset = offset;
	nPages = 0;
	PageStat = PS_FORW;
}

/*
 *	IsStopPage
 */
static BOOL IsStopPage()
{
	switch (PageStat) {
	case PS_FORW:
		if (workOffset >= nText)
			return TRUE;
		break;
	case PS_BACK:
		if (workOffset >= StartOffset)
			return TRUE;
		break;
	}

	return FALSE;
}

/*
 *	SetupPage
 */
static BOOL SetupPage()
{
	int		x, y, w;
	int		wk;
	u8		c, c2;
	u8		*pt;
	static u32	ticks;
	PSCRNOBJ	pso;

	if (PageStat == PS_DONE)
		return TRUE;

	if (GetKeyFunc(GetKey()) == KF_BACK)
		return FALSE;

	if (&pPage[workPage] >= pPageEnd) {
		// TODO: Error handling
		pso = GetScrnObject();
		XfontSelectFontObject(FO_SMALL);
		SetWinTitle("No Memory.");
		return FALSE;
	}

	XfontSelectFontObject(FO_NORMAL);
	wk = XfontGetMaxCharWidth(0x81);

	pPage[workPage] = workOffset;

	for (y = rcText.y; y < (rcText.y + rcText.h - Hnormal); y += Hnormal) {
		x = rcText.x;
		while (x < (rcText.x + rcText.w - wk)) {
			if (IsStopPage())
				goto nextpage;
			pt = &pText[workOffset];
			c = *pt;
			workOffset += GETCHARLENGTH(c);
			switch (c) {
			default:
				x += XfontGetCharWidth(&pt);
				break;
			case 0x0d:
				x = rcText.x;
				pt++;
				break;
			case 0x0a:
				pt++;
				goto nextline;
			}
		}
	nextline:
		;
	}

nextpage:
	workPage++;
	switch (PageStat) {
	case PS_FORW:
		nPages = workPage;
		if (IsStopPage()) {
			if (StartPage) {
				workPage = 0;
				workOffset = 0;
				PageStat = PS_BACK;
			} else
				PageStat = PS_DONE;
		}
		break;
	case PS_BACK:
		if (IsStopPage())
			PageStat = PS_DONE;
		break;
	}

	if (PageStat == PS_DONE || (ClockTick > ticks && nPages > 1)) {
		ticks = ClockTick + 500;
		ViewPageNumber();
	}

	return TRUE;
}

/*
 *	IsPageReady
 */
static BOOL IsPageReady(u32 pageno)
{
	switch (PageStat) {
	case PS_DONE:
		return TRUE;
	case PS_BACK:
		if (pageno < workPage)
			return TRUE;
		if (pageno >= StartPage && pageno < nPages)
			return TRUE;
		break;
	case PS_FORW:
		if (pageno >= StartPage && pageno < workPage)
			return TRUE;
		break;
	}

	return FALSE;
}

/*
 *	GetPageOffset
 */
static u32 GetPageOffset(u32 pageno)
{
	if (!nText)
		return INVALID_PAGENO;

	while (!IsPageReady(pageno))
		if (!SetupPage())
			return INVALID_PAGENO;

	return pPage[pageno];
}

/*
 *	PutText
 */
static void PutText(int pageno)
{
	int		x, y, w, len;
	int		wk;
	u8		c;
	u32		offset;
	u8		*pt, *pe;

	offset = GetPageOffset(pageno);
	if (offset == INVALID_PAGENO)
		return;
	ViewBookmark();
	ViewPageNumber();

	XfontSelectFontObject(FO_NORMAL);
	wk = XfontGetMaxCharWidth(0x81);

	pt = &pText[offset];
	pe = &pText[nText];
	x = rcText.x;
	len = 0;
	for (y = rcText.y; y < (rcText.y + rcText.h - Hnormal); y += Hnormal) {
		x = rcText.x;
		len = 0;
		while (x < (rcText.x + rcText.w - wk)) {
			if (pt >= pe)
				goto out;
			c = *pt;
			switch (c) {
			default:
				w = XfontPutChar(x, y, &pt);
				x += w;
				len += w;
				break;
			case 0x0d:
				x = rcText.x;
				pt++;
				break;
			case 0x0a:
				pt++;
				goto nextline;
			}
		}
	nextline:
		if (len < rcText.w)
			EraseScrn(rcText.x + len, y, rcText.w - len, Hnormal);
	}

out:
	if (y < (rcText.y + rcText.h)) {
		if (x > rcText.x && len < rcText.w &&
			y < (rcText.y + rcText.h - Hnormal))
		{
			EraseScrn(rcText.x + len, y, rcText.w - len, Hnormal);
			y += Hnormal;
		}
		EraseScrn(rcText.x, y, rcText.w, (rcText.y + rcText.h) - y);
	}
}

/*
 *	RefreshView
 */
static void RefreshView(BOOL force)
{
	CREATEWIN	cw;

	if (force) {
		memset(&cw, 0, sizeof cw);
		cw.xStartTitle = bmBookmark.width;
		CreateWinFrame(&cw);
		InitPositions();
		SetWinTitle(pTextName);
	}
}

/*
 *	PageUp
 */
static BOOL PageUp(BOOL *pdirty)
{
	if (CurPage > 0) {
		CurPage--;
		*pdirty = TRUE;
		return TRUE;
	} else {
		while (PageStat != PS_DONE) {
			if (!SetupPage())
				return FALSE;
		}
		if (nPages <= 1) {
			*pdirty = FALSE;
			return TRUE;
		}
		CurPage = nPages - 1;
		*pdirty = TRUE;
		return TRUE;
	}
}

/*
 *	PageDown
 */
static BOOL PageDown(BOOL *pdirty)
{
	if (CurPage >= nPages) {
		while (!IsPageReady(CurPage)) {
			if (!SetupPage())
				return FALSE;
		}
	}
	if (PageStat == PS_DONE) {
		if (nPages <= 1) {
			*pdirty = FALSE;
			return TRUE;
		}
		if (CurPage + 1 >= nPages)
			CurPage = 0;
		else
			CurPage++;
	} else
		CurPage++;
	*pdirty = TRUE;
	return TRUE;
}

/*
 *	ViewTextGoto
 */
int ViewTextGoto(int pageno)
{
	CurPage = pageno;

	return LB_CANCEL;
}

/*
 *	ViewText
 */
void ViewText(PDIRENT pdir)
{
	int		key;
	BOOL		dirty;
	BOOL		inrepeat;
	u32		offset;
	BMPARAMS	parms;
	int		mode;
	PKEYMAP		poldkey, pkey;
	int		res;

	pText = pdir->fileaddr + pdir->namesize;
	nText = pdir->filesize;
	pTextName = pdir->fileaddr;

	poldkey = GetBaseKeyMap();
	pkey = GetTextKeyMap();
	SetBaseKeyMap(pkey);

setfont:
	XfontSelectFontObject(FO_NORMAL);
	parms.pdir = pdir;
	parms.fontname = XfontGetFontName();
	parms.fontsize = XfontGetCharHeight();
	parms.scrndir = GetScrnMode();
	if (FindBookMark(&parms)) {
		CurPage = parms.pageno;
		offset = parms.offset;
		BookmarkPage = CurPage;
	} else {
		CurPage = 0;
		offset = 0;
		BookmarkPage = INVALID_PAGENO;
	}

	InitPage(offset);

restart:
	RefreshView(TRUE);

	dirty = FALSE;
	inrepeat = FALSE;

	PutText(CurPage);

	for (;;) {
		while ((key = GetKey()) == 0) {
			if (!SetupPage())
				return;
			if (inrepeat) {
				PutText(CurPage);
				inrepeat = FALSE;
			}
		}

		switch (GetKeyFunc(key)) {
		case KF_BACK:
			goto out;
		case KF_NEXTPAGE:
			if (!PageDown(&dirty))
				goto out;
			break;
		case KF_PREVPAGE:
			if (!PageUp(&dirty))
				goto out;
			break;
		case KF_BOOKMARK:
			XfontSelectFontObject(FO_NORMAL);
			parms.pdir = pdir;
			parms.pageno = CurPage;
			parms.offset = GetPageOffset(CurPage);
			parms.fontname = XfontGetFontName();
			parms.fontsize = XfontGetCharHeight();
			parms.scrndir = GetScrnMode();
			if (BuildBookMark(&parms)) {
				BookmarkPage = CurPage;
				ViewBookmark();
			}
			break;
		case KF_OPTIONS:
			SetBaseKeyMap(poldkey);
			res = TextOptions();
			SetBaseKeyMap(pkey);
			if (res >= 0)
				goto setfont;
			goto restart;
		}

		if (dirty) {
			if (key & KEY_REPEAT) {
				ViewBookmark();
				ViewPageNumber();
				Wait(max(10, 2000 / nPages));
				inrepeat = TRUE;
			} else
				PutText(CurPage);
			dirty = FALSE;
		}
	}

out:
	SetBaseKeyMap(poldkey);
}

