/*
 *	heap.c
 */

#include <gba.h>
#include <heap.h>

//====	EWRAM Heap

static u8	*HeapTop;
static int	HeapSize;

/*
 *	HeapInit
 */
void HeapInit()
{
	HeapTop = (u8*)EWRAM_ADDR;
	HeapSize = EWRAM_SIZE;
}

/*
 *	HeapAlloc
 */
void *HeapAlloc(int size)
{
	void	*p;

	if (size > HeapSize)
		return NULL;
	p = HeapTop;
	HeapTop += size;
	HeapSize -= size;

	return p;
}

/*
 *	GetFreeHeap
 */
int GetFreeHeap()
{
	return HeapSize;
}

//====	IWRAM Heap

#define	IWRAM_STACKSIZE	0x2000
#define	IWHEAP_LIMIT	(IWRAM_ADDR + IWRAM_SIZE - IWRAM_STACKSIZE)

#define ROUNDUP32(p)	(((int)(p) + 3) & ~3)

static u8	*iwHeapTop;
static u8	*iwHeapEnd;

/*
 *	iHeapInit
 */
void iHeapInit()
{
	extern int __bss_end__;

	iwHeapTop = (u8 *)&__bss_end__;
	iwHeapEnd = (u8 *)IWHEAP_LIMIT;
}

/*
 *	iGetHeap
 */
void iGetHeap(u8 **pptop, u8 **ppend)
{
	*pptop = iwHeapTop;
	*ppend = iwHeapEnd;
}

/*
 *	iSetHeapTop
 */
void iSetHeapTop(u8 *ptop)
{
	iwHeapTop = (u8 *)ROUNDUP32(ptop);
}

