/*
 *	xindex.c
 */

#include <xfont.h>
#include <gbafs.h>
#include <chartype.h>

#define INVALID_CODE		0x8145
#define INVALID_INDEX		5

#define MAX_CCT			8

static u16	*aCCT[MAX_CCT];
static int	nCCT;

/*
 *	IsValidCCT
 */
static BOOL IsValidCCT(PXCCTHEAD pxc)
{
	return memcmp(pxc->sig, XCCTH_SIG, sizeof(pxc->sig)) == 0 &&
		pxc->ver == XCCTH_VER &&
		pxc->size == sizeof(*pxc);
}

/*
 *	XfontInitCCT
 */
void XfontInitCCT()
{
	PDIRENT		pdir;
	PDIRENT		pd, pe;
	PXCCTHEAD	pxc;
	int		i;

	nCCT = 0;
	memset(aCCT, 0, sizeof aCCT);

	if (!(pdir = FsGetDirEnt((u8 *)FONT_PATH)))
		return;

	pd = (PDIRENT)(pdir->fileaddr + pdir->namesize);
	pe = (PDIRENT)((u8 *)pd + pdir->filesize);
	for (; pd < pe; pd++) {
		if (pd->attr & DA_DIR)
			continue;
		pxc = (PXCCTHEAD)(pd->fileaddr + pd->namesize);
		if (!IsValidCCT(pxc))
			continue;

		i = pxc->index;
		if (i >= MAX_CCT)
			continue;

		aCCT[i] = (u16 *)((u8 *)pxc + sizeof(*pxc));
		if (i + 1 > nCCT && i < MAX_CCT)
			nCCT = i + 1;
	}
}

/*
 *	XfontGetCCT
 */
void *XfontGetCCT(int index)
{
	if (index >= 0 && index < nCCT && aCCT[index])
		return aCCT[index];
	return NULL;
}

/*
 *	XfontKanjiCode2Index
 */
attriwram
int XfontKanjiCode2Index(PXFONTOBJ pxo, u16 code)
{
	u8		*pcct;
	PXCCTENT	pe;
	PXCCT		pc;
	u16		*p1;
	int		i0, i1, i2, i3;
	int		i;

	if (!(pcct = (u8 *)(pxo->pcct)))
		return INVALID_INDEX;

	if (GETCHARTYPE(HIBYTE(code)) != CT_KANJI)
		return INVALID_INDEX;

	/*
	 *	level 1
	 */

	i0 = HIBYTE(code) >> 5;
	i1 = HIBYTE(code) & 0x1f;
	if (i0 == 4)
		i1 = ((u16 *)pcct)[i1];
	else
		i1 = ((u16 *)pcct)[i1 + 32];
	if (!i1)
		return INVALID_INDEX;
	p1 = (u16 *)(pcct + i1);

	/*
	 *	level 2
	 */

	i2 = LOBYTE(code) >> 6;
	i2 = p1[i2];
	if (!i2)
		return INVALID_INDEX;

	pe = (PXCCTENT)(pcct + i2);
	pc = (PXCCT)(pcct + i2 + sizeof(*pe));

	/*
	 *	level 3
	 */

	i3 = LOBYTE(code) & 0x3f;
	for (i = 0; i < pe->count; i++, pc++) {
		if (i3 >= pc->start && i3 <= pc->end)
			return pc->offset + (i3 - pc->start);
	}

	return INVALID_INDEX;
}

static const u16 KanaMap[] = {
	0x8140, 0x8142, 0x8175, 0x8176, 0x8141, 0x8145, 0x8392, 0x8340, // a0-7
	0x8342, 0x8344, 0x8346, 0x8348, 0x8383, 0x8385, 0x8387, 0x8362, // a8-f
	0x815b, 0x8341, 0x8343, 0x8345, 0x8347, 0x8349, 0x834a, 0x834c, // b0-7
	0x834e, 0x8350, 0x8352, 0x8354, 0x8356, 0x8358, 0x835a, 0x835c, // b8-f
	0x835e, 0x8360, 0x8363, 0x8365, 0x8367, 0x8369, 0x836a, 0x836b, // c0-7
	0x836c, 0x836d, 0x836e, 0x8371, 0x8374, 0x8377, 0x837a, 0x837d, // c8-f
	0x837e, 0x8380, 0x8381, 0x8382, 0x8384, 0x8386, 0x8388, 0x8389, // d0-7
	0x838a, 0x838b, 0x838c, 0x838d, 0x838f, 0x8393, 0x814a, 0x814b, // d8-f
};

/*
 *	XfontKanaCode2Index
 */
attriwram
int XfontKanaCode2Index(PXFONTOBJ pxo, u16 code)
{
	return XfontKanjiCode2Index(pxo,
		HIBYTE(code) ? code : KanaMap[code - 0xa0]);
}

/*
 *	XfontAnsiCode2Index
 */
attriwram
int XfontAnsiCode2Index(PXFONTOBJ pxo, u16 code)
{
	PXFONTHEAD	pxf = pxo->pxf;

	if (code == 0x09)	// TAB code
		code = 0x20;
	else if (code < pxf->scode || code > pxf->ecode)
		code = 0;

	return code - pxf->scode;
}

